/* =========================================================
 * bootstrap-modal.js v2.0.4
 * http://twitter.github.com/bootstrap/javascript.html#modals
 * =========================================================
 * Copyright 2012 Twitter, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================= */


!function ($) {

  "use strict"; // jshint ;_;


 /* MODAL CLASS DEFINITION
  * ====================== */

  var Modal = function (content, options) {
    this.options = options
    this.$element = $(content)
      .delegate('[data-dismiss="modal"]', 'click.dismiss.modal', $.proxy(this.hide, this))
  }

  Modal.prototype = {

      constructor: Modal

    , toggle: function () {
        return this[!this.isShown ? 'show' : 'hide']()
      }

    , show: function () {
        var that = this
          , e = $.Event('show')

        this.$element.trigger(e)

        if (this.isShown || e.isDefaultPrevented()) return

        $('body').addClass('modal-open')

        this.isShown = true

        escape.call(this)
        backdrop.call(this, function () {
          var transition = $.support.transition && that.$element.hasClass('fade')

          if (!that.$element.parent().length) {
            that.$element.appendTo(document.body) //don't move modals dom position
          }

          that.$element
            .show()

          if (transition) {
            that.$element[0].offsetWidth // force reflow
          }

          that.$element.addClass('in')

          transition ?
            that.$element.one($.support.transition.end, function () { that.$element.trigger('shown') }) :
            that.$element.trigger('shown')

        })
      }

    , hide: function (e) {
        e && e.preventDefault()

        var that = this

        e = $.Event('hide')

        this.$element.trigger(e)

        if (!this.isShown || e.isDefaultPrevented()) return

        this.isShown = false

        $('body').removeClass('modal-open')

        escape.call(this)

        this.$element.removeClass('in')

        $.support.transition && this.$element.hasClass('fade') ?
          hideWithTransition.call(this) :
          hideModal.call(this)
      }

  }


 /* MODAL PRIVATE METHODS
  * ===================== */

  function hideWithTransition() {
    var that = this
      , timeout = setTimeout(function () {
          that.$element.off($.support.transition.end)
          hideModal.call(that)
        }, 500)

    this.$element.one($.support.transition.end, function () {
      clearTimeout(timeout)
      hideModal.call(that)
    })
  }

  function hideModal(that) {
    this.$element
      .hide()
      .trigger('hidden')

    backdrop.call(this)
  }

  function backdrop(callback) {
    var that = this
      , animate = this.$element.hasClass('fade') ? 'fade' : ''

    if (this.isShown && this.options.backdrop) {
      var doAnimate = $.support.transition && animate

      this.$backdrop = $('<div class="modal-backdrop ' + animate + '" />')
        .appendTo(document.body)

      if (this.options.backdrop != 'static') {
        this.$backdrop.click($.proxy(this.hide, this))
      }

      if (doAnimate) this.$backdrop[0].offsetWidth // force reflow

      this.$backdrop.addClass('in')

      doAnimate ?
        this.$backdrop.one($.support.transition.end, callback) :
        callback()

    } else if (!this.isShown && this.$backdrop) {
      this.$backdrop.removeClass('in')

      $.support.transition && this.$element.hasClass('fade')?
        this.$backdrop.one($.support.transition.end, $.proxy(removeBackdrop, this)) :
        removeBackdrop.call(this)

    } else if (callback) {
      callback()
    }
  }

  function removeBackdrop() {
    this.$backdrop.remove()
    this.$backdrop = null
  }

  function escape() {
    var that = this
    if (this.isShown && this.options.keyboard) {
      $(document).on('keyup.dismiss.modal', function ( e ) {
        e.which == 27 && that.hide()
      })
    } else if (!this.isShown) {
      $(document).off('keyup.dismiss.modal')
    }
  }


 /* MODAL PLUGIN DEFINITION
  * ======================= */

  $.fn.modal = function (option) {
    return this.each(function () {
      var $this = $(this)
        , data = $this.data('modal')
        , options = $.extend({}, $.fn.modal.defaults, $this.data(), typeof option == 'object' && option)
      if (!data) $this.data('modal', (data = new Modal(this, options)))
      if (typeof option == 'string') data[option]()
      else if (options.show) data.show()
    })
  }

  $.fn.modal.defaults = {
      backdrop: true
    , keyboard: true
    , show: true
  }

  $.fn.modal.Constructor = Modal


 /* MODAL DATA-API
  * ============== */

  $(function () {
    $('body').on('click.modal.data-api', '[data-toggle="modal"]', function ( e ) {
      var $this = $(this), href
        , $target = $($this.attr('data-target') || (href = $this.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '')) //strip for ie7
        , option = $target.data('modal') ? 'toggle' : $.extend({}, $target.data(), $this.data())

      e.preventDefault()
      $target.modal(option)
    })
  })

}(window.jQuery);/* ===========================================================
 * bootstrap-tooltip.js v2.0.4
 * http://twitter.github.com/bootstrap/javascript.html#tooltips
 * Inspired by the original jQuery.tipsy by Jason Frame
 * ===========================================================
 * Copyright 2012 Twitter, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================== */


!function ($) {

	"use strict"; // jshint ;_;


 /* TOOLTIP PUBLIC CLASS DEFINITION
	* =============================== */

	var Tooltip = function (element, options) {
		this.init('tooltip', element, options)
	}

	Tooltip.prototype = {

		constructor: Tooltip

	, init: function (type, element, options) {
			var eventIn
				, eventOut

			this.type = type
			this.$element = $(element)
			this.options = this.getOptions(options)
			this.enabled = true

			if (this.options.trigger != 'manual') {
				eventIn  = this.options.trigger == 'hover' ? 'mouseenter' : 'focus'
				eventOut = this.options.trigger == 'hover' ? 'mouseleave' : 'blur'
				this.$element.on(eventIn, this.options.selector, $.proxy(this.enter, this))
				this.$element.on(eventOut, this.options.selector, $.proxy(this.leave, this))
			}

			this.options.selector ?
				(this._options = $.extend({}, this.options, { trigger: 'manual', selector: '' })) :
				this.fixTitle()
		}

	, getOptions: function (options) {
			options = $.extend({}, $.fn[this.type].defaults, options, this.$element.data())

			if (options.delay && typeof options.delay == 'number') {
				options.delay = {
					show: options.delay
				, hide: options.delay
				}
			}

			return options
		}

	, enter: function (e) {
			var self = $(e.currentTarget)[this.type](this._options).data(this.type)

			if (!self.options.delay || !self.options.delay.show) return self.show()

			clearTimeout(this.timeout)
			self.hoverState = 'in'
			this.timeout = setTimeout(function() {
				if (self.hoverState == 'in') self.show()
			}, self.options.delay.show)
		}

	, leave: function (e) {
			var self = $(e.currentTarget)[this.type](this._options).data(this.type)

			if (this.timeout) clearTimeout(this.timeout)
			if (!self.options.delay || !self.options.delay.hide) return self.hide()

			self.hoverState = 'out'
			this.timeout = setTimeout(function() {
				if (self.hoverState == 'out') self.hide()
			}, self.options.delay.hide)
		}

	, show: function () {
			var $tip
				, inside
				, pos
				, actualWidth
				, actualHeight
				, placement
				, tp

			if (this.hasContent() && this.enabled) {
				$tip = this.tip()
				this.setContent()

				if (this.options.animation) {
					$tip.addClass('fade')
				}

				placement = typeof this.options.placement == 'function' ?
					this.options.placement.call(this, $tip[0], this.$element[0]) :
					this.options.placement

				inside = /in/.test(placement)

				$tip
					.remove()
					.css({ top: 0, left: 0, display: 'block' })
					.appendTo(inside ? this.$element : document.body)

				pos = this.getPosition(inside)

				actualWidth = $tip[0].offsetWidth
				actualHeight = $tip[0].offsetHeight

				switch (inside ? placement.split(' ')[1] : placement) {
					case 'bottom':
						tp = {top: pos.top + pos.height, left: pos.left + pos.width / 2 - actualWidth / 2}
						break
					case 'top':
						tp = {top: pos.top - actualHeight, left: pos.left + pos.width / 2 - actualWidth / 2}
						break
					case 'left':
						tp = {top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left - actualWidth}
						break
					case 'right':
						tp = {top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left + pos.width}
						break
				}

				$tip
					.css(tp)
					.addClass(placement)
					.addClass('in')
			}
		}

	, isHTML: function(text) {
			// html string detection logic adapted from jQuery
			return typeof text != 'string'
				|| ( text.charAt(0) === "<"
					&& text.charAt( text.length - 1 ) === ">"
					&& text.length >= 3
				) || /^(?:[^<]*<[\w\W]+>[^>]*$)/.exec(text)
		}

	, setContent: function () {
			var $tip = this.tip()
				, title = this.getTitle()

			$tip.find('.tooltip-inner')[this.isHTML(title) ? 'html' : 'text'](title)
			$tip.removeClass('fade in top bottom left right')
		}

	, hide: function () {
			var that = this
				, $tip = this.tip()

			$tip.removeClass('in')

			function removeWithAnimation() {
				var timeout = setTimeout(function () {
					$tip.off($.support.transition.end).remove()
				}, 500)

				$tip.one($.support.transition.end, function () {
					clearTimeout(timeout)
					$tip.remove()
				})
			}

			$.support.transition && this.$tip.hasClass('fade') ?
				removeWithAnimation() :
				$tip.remove()
		}

	, fixTitle: function () {
			var $e = this.$element
			if ($e.attr('title') || typeof($e.attr('data-original-title')) != 'string') {
				$e.attr('data-original-title', $e.attr('title') || '').removeAttr('title')
			}
		}

	, hasContent: function () {
			return this.getTitle()
		}

	, getPosition: function (inside) {
			return $.extend({}, (inside ? {top: 0, left: 0} : this.$element.offset()), {
				width: this.$element[0].offsetWidth
			, height: this.$element[0].offsetHeight
			})
		}

	, getTitle: function () {
			var title
				, $e = this.$element
				, o = this.options

			title = $e.attr('data-original-title')
				|| (typeof o.title == 'function' ? o.title.call($e[0]) :	o.title)

			return title
		}

	, tip: function () {
			return this.$tip = this.$tip || $(this.options.template)
		}

	, validate: function () {
			if (!this.$element[0].parentNode) {
				this.hide()
				this.$element = null
				this.options = null
			}
		}

	, enable: function () {
			this.enabled = true
		}

	, disable: function () {
			this.enabled = false
		}

	, toggleEnabled: function () {
			this.enabled = !this.enabled
		}

	, toggle: function () {
			this[this.tip().hasClass('in') ? 'hide' : 'show']()
		}

	}


 /* TOOLTIP PLUGIN DEFINITION
	* ========================= */

	$.fn.tooltip = function ( option ) {
		return this.each(function () {
			var $this = $(this)
				, data = $this.data('tooltip')
				, options = typeof option == 'object' && option
			if (!data) $this.data('tooltip', (data = new Tooltip(this, options)))
			if (typeof option == 'string') data[option]()
		})
	}

	$.fn.tooltip.Constructor = Tooltip

	$.fn.tooltip.defaults = {
		animation: true
	, placement: 'top'
	, selector: false
	, template: '<div class="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>'
	, trigger: 'hover'
	, title: ''
	, delay: 0
	}

}(window.jQuery);
/* ===========================================================
 * bootstrap-popover.js v2.0.4
 * http://twitter.github.com/bootstrap/javascript.html#popovers
 * ===========================================================
 * Copyright 2012 Twitter, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =========================================================== */


!function ($) {

	"use strict"; // jshint ;_;


 /* POPOVER PUBLIC CLASS DEFINITION
	* =============================== */

	var Popover = function ( element, options ) {
		this.init('popover', element, options)
	}


	/* NOTE: POPOVER EXTENDS BOOTSTRAP-TOOLTIP.js
		 ========================================== */

	Popover.prototype = $.extend({}, $.fn.tooltip.Constructor.prototype, {

		constructor: Popover

	, setContent: function () {
			var $tip = this.tip()
				, title = this.getTitle()
				, content = this.getContent()

			$tip.find('.popover-title')[this.isHTML(title) ? 'html' : 'text'](title)
			$tip.find('.popover-content > *')[this.isHTML(content) ? 'html' : 'text'](content)

			$tip.removeClass('fade top bottom left right in')
		}

	, hasContent: function () {
			return this.getTitle() || this.getContent()
		}

	, getContent: function () {
			var content
				, $e = this.$element
				, o = this.options

			content = $e.attr('data-content')
				|| (typeof o.content == 'function' ? o.content.call($e[0]) :	o.content)

			return content
		}

	, tip: function () {
			if (!this.$tip) {
				this.$tip = $(this.options.template)
			}
			return this.$tip
		}

	})


 /* POPOVER PLUGIN DEFINITION
	* ======================= */

	$.fn.popover = function (option) {
		return this.each(function () {
			var $this = $(this)
				, data = $this.data('popover')
				, options = typeof option == 'object' && option
			if (!data) $this.data('popover', (data = new Popover(this, options)))
			if (typeof option == 'string') data[option]()
		})
	}

	$.fn.popover.Constructor = Popover

	$.fn.popover.defaults = $.extend({} , $.fn.tooltip.defaults, {
		placement: 'right'
	, content: ''
	, template: '<div class="popover"><div class="arrow"></div><div class="popover-inner"><h3 class="popover-title"></h3><div class="popover-content"><p></p></div></div></div>'
	})

}(window.jQuery);
/* ===================================================
 * bootstrap-transition.js v2.0.4
 * http://twitter.github.com/bootstrap/javascript.html#transitions
 * ===================================================
 * Copyright 2012 Twitter, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================== */


!function ($) {

  $(function () {

    "use strict"; // jshint ;_;


    /* CSS TRANSITION SUPPORT (http://www.modernizr.com/)
     * ======================================================= */

    $.support.transition = (function () {

      var transitionEnd = (function () {

        var el = document.createElement('bootstrap')
          , transEndEventNames = {
               'WebkitTransition' : 'webkitTransitionEnd'
            ,  'MozTransition'    : 'transitionend'
            ,  'OTransition'      : 'oTransitionEnd'
            ,  'msTransition'     : 'MSTransitionEnd'
            ,  'transition'       : 'transitionend'
            }
          , name

        for (name in transEndEventNames){
          if (el.style[name] !== undefined) {
            return transEndEventNames[name]
          }
        }

      }())

      return transitionEnd && {
        end: transitionEnd
      }

    })()

  })

}(window.jQuery);var WebDeveloper = WebDeveloper || {};

WebDeveloper.Generated							= WebDeveloper.Generated || {};
WebDeveloper.Generated.cookie				= null;
WebDeveloper.Generated.storedLocale = null;

// Handles the cookie session setting being changed
WebDeveloper.Generated.changeSession = function()
{
	var session = $(this);

	// If the session setting is checked
	if(session.prop("checked"))
	{
		$("#cookie-expires").val("").prop("disabled", true);
	}
	else
	{
		$("#cookie-expires").val(WebDeveloper.Cookies.getDateTomorrow()).prop("disabled", false);
	}
};

// Handles clicking edit
WebDeveloper.Generated.clickEdit = function()
{
	var editButton = $(this);

	// If the cookie can be edited
	if(editButton.hasClass("btn-primary"))
	{
		WebDeveloper.Generated.showEditDialog(editButton.parent());
	}
	else
	{
		editButton.popover("toggle");
	}
};

// Deletes a cookie
WebDeveloper.Generated.deleteCookie = function()
{
	var cookie = WebDeveloper.Generated.populateCookieFromElement(WebDeveloper.Generated.cookie);

	WebDeveloper.Cookies.deleteCookie(cookie);

	WebDeveloper.Generated.cookie.slideUp(WebDeveloper.Generated.animationSpeed, function() { WebDeveloper.Generated.cookie.remove(); });
	$("#delete-dialog").modal("hide");
	WebDeveloper.Generated.cookie.before('<div class="alert alert-success">' + WebDeveloper.Generated.storedLocale.cookieDeleted.replace("%S", "<strong>" + cookie.name + "</strong>") + "</div>");
};

// Edits a cookie
WebDeveloper.Generated.editCookie = function()
{
	// If the dialog is valid
	if(WebDeveloper.Generated.validateEditDialog())
	{
		var newCookie = WebDeveloper.Generated.populateCookieFromDialog();
		var oldCookie = WebDeveloper.Generated.populateCookieFromElement(WebDeveloper.Generated.cookie);

		WebDeveloper.Cookies.deleteCookie(oldCookie);
		WebDeveloper.Cookies.addCookie(newCookie);
		WebDeveloper.Generated.populateElementFromCookie(WebDeveloper.Generated.cookie, newCookie);

		WebDeveloper.Generated.cookie.prepend('<div class="alert alert-success">' + WebDeveloper.Generated.storedLocale.cookieEdited.replace("%S", "<strong>" + oldCookie.name + "</strong>") + "</div>");
		$("#edit-dialog").modal("hide");
	}
};

// Generates the commands
WebDeveloper.Generated.generateCommands = function(cookie, locale)
{
	var commands	 = "";
	var cookieHost = cookie.host;

	commands += '<button class="web-developer-delete btn btn-danger"><i class="icon-trash"></i> ' + locale.deleteConfirmation + "</button>";

	// If the cookie is HTTP onlu
	if(cookie.httpOnly)
	{
		commands += '<button class="web-developer-edit btn" data-content="' + locale.cannotEditHTTPOnlyCookies + '" data-title="' + locale.cannotEdit + '" data-trigger="manual"><i class="icon-pencil"></i> ' + locale.edit + "</button>";
	}
	else if(!WebDeveloper.Cookies.canEditLocalCookie() && (cookieHost == "localhost" || cookieHost == ".localhost"))
	{
		commands += '<button class="web-developer-edit btn" data-content=' + "'" + locale.cannotEditLocalhostCookies + "'" + ' data-title="' + locale.cannotEdit + '" data-trigger="manual"><i class="icon-pencil"></i> ' + locale.edit + "</button>";
	}
	else
	{
		commands += '<button class="web-developer-edit btn btn-primary"><i class="icon-pencil"></i> ' + locale.edit + "</button>";
	}

	return commands;
};

// Initializes the page with data
WebDeveloper.Generated.initialize = function(data, locale)
{
	var container						= null;
	var content							= $("#content");
	var contentDocument			= null;
	var cookieInformation		= locale.cookieInformation;
	var documents						= data.documents;
	var cookie							= null;
	var cookieDescription		= null;
	var cookieElement				= null;
	var cookieExpires				= null;
	var cookieName					= null;
	var cookies							= null;
	var cookiesCounter			= 1;
	var cookiesDropdown			= $("#cookies-dropdown");
	var cookiesDropdownMenu = $(".dropdown-menu", cookiesDropdown);
	var cookiesLength				= null;
	var deleteDialog				= $("#delete-dialog");
	var editDialog					= $("#edit-dialog");
	var expiresDescription	= null;
	var httpOnlyDescription = null;
	var secureDescription		= null;
	var table								= null;
	var tableBody						= null;

	WebDeveloper.Generated.emptyContent();
	WebDeveloper.Generated.localizeHeader(locale);
	WebDeveloper.Generated.setPageTitle(cookieInformation, data, locale);

	$(".dropdown-toggle", cookiesDropdown).prepend(locale.cookies);

	// Loop through the documents
	for(var i = 0, l = documents.length; i < l; i++)
	{
		contentDocument		= documents[i];
		cookieDescription = locale.cookies.toLowerCase();
		cookies						= contentDocument.cookies;
		cookiesLength			= cookies.length;

		// If there is only one cookie
		if(cookiesLength == 1)
		{
			cookieDescription = locale.cookie.toLowerCase();
		}

		WebDeveloper.Generated.addDocument(contentDocument.url, i, cookieDescription, cookiesLength);

		// If there are cookies
		if(cookiesLength > 0)
		{
			container = WebDeveloper.Generated.generateDocumentContainer();

			// Loop through the cookies
			for(var j = 0; j < cookiesLength; j++)
			{
				cookie							= cookies[j];
				cookieElement				= $('<div id="cookie-' + cookiesCounter + '" class="web-developer-cookie"></div>');
				cookieExpires				= cookie.expires;
				cookieName					= cookie.name;
				expiresDescription	= locale.atEndOfSession;
				httpOnlyDescription = locale.no;
				secureDescription		= locale.no;
				table								= $('<table class="table table-bordered table-striped"></table>');
				tableBody						= $("<tbody></tbody>");

				table.append('<thead><tr><th>' + locale.property + '</th><th>' + locale.value + '</th></tr></thead>');
				tableBody.append('<tr><td>' + locale.name + '</td><td class="web-developer-name">' + cookieName + '</td></tr>');
				tableBody.append('<tr><td>' + locale.value + '</td><td class="web-developer-value">' + cookie.value + '</td></tr>');
				tableBody.append('<tr><td>' + locale.host + '</td><td class="web-developer-host">' + cookie.host + '</td></tr>');
				tableBody.append('<tr><td>' + locale.path + '</td><td class="web-developer-path">' + cookie.path + '</td></tr>');

				// If the cookie has an expiration
				if(cookieExpires)
				{
					expiresDescription = new Date(cookieExpires * 1000).toUTCString();
				}

				tableBody.append('<tr><td>' + locale.expires + '</td><td class="web-developer-expires">' + expiresDescription + '</td></tr>');

				// If the cookie is secure
				if(cookie.secure)
				{
					secureDescription = locale.yes;
				}

				tableBody.append('<tr><td>' + locale.secure + '</td><td class="web-developer-secure">' + secureDescription + '</td></tr>');

				// If the cookie is HttpOnly
				if(cookie.httpOnly)
				{
					httpOnlyDescription = locale.yes;
				}

				tableBody.append('<tr><td>' + locale.httpOnly + '</td><td>' + httpOnlyDescription + '</td></tr>');

				table.append(tableBody);
				cookieElement.append(table).append(WebDeveloper.Generated.generateCommands(cookie, locale));
				container.append(cookieElement).append('<div class="web-developer-separator"></div>');
				content.append(container);
				cookiesDropdownMenu.append('<li><a href="#cookie-' + cookiesCounter + '">' + cookieName + "</a></li>");

				cookiesCounter++;
			}
		}
		else
		{
			WebDeveloper.Generated.addSeparator();
		}
	}

	WebDeveloper.Generated.storedLocale = locale;

	$("#cookie-secure").after(locale.secureCookie);
	$("#cookie-session").after(locale.sessionCookie).on("change", WebDeveloper.Generated.changeSession);
	$(".btn-danger", deleteDialog).append(locale.deleteLabel).on("click", WebDeveloper.Generated.deleteCookie);
	$(".btn-primary", editDialog).append(locale.save).on("click", WebDeveloper.Generated.editCookie);
	$('button[data-dismiss="modal"]', deleteDialog).html(locale.cancel);
	$('button[data-dismiss="modal"]', editDialog).html(locale.cancel);
	$(".web-developer-delete").on("click", WebDeveloper.Generated.showDeleteDialog);
	$(".web-developer-edit").on("click", WebDeveloper.Generated.clickEdit);
	$('[for="cookie-expires"]').html(locale.expires);
	$('[for="cookie-host"]').html(locale.host);
	$('[for="cookie-name"]').html(locale.name);
	$('[for="cookie-path"]').html(locale.path);
	$('[for="cookie-value"]').html(locale.value);

	WebDeveloper.Generated.initializeCommonElements();
};

// Populates a cookie from a dialog
WebDeveloper.Generated.populateCookieFromDialog = function()
{
	var cookie = {};

	cookie.host  = $("#cookie-host").val();
	cookie.name  = $("#cookie-name").val();
	cookie.path  = $("#cookie-path").val();
	cookie.value = $("#cookie-value").val();

	// If the cookie is secure
	if($("#cookie-secure").prop("checked"))
	{
		cookie.secure = true;
	}

	// If the cookie is a session cookie
	if($("#cookie-session").prop("checked"))
	{
		cookie.session = true;
	}
	else
	{
		cookie.expires = $("#cookie-expires").val();
	}

	return cookie;
};

// Populates a cookie from an element
WebDeveloper.Generated.populateCookieFromElement = function(cookieElement)
{
	var cookie = {};

	cookie.host = $(".web-developer-host", cookieElement).text();
	cookie.name = $(".web-developer-name", cookieElement).text();
	cookie.path = $(".web-developer-path", cookieElement).text();

	// If the cookie is secure
	if($(".web-developer-secure", cookieElement).text() == WebDeveloper.Generated.storedLocale.yes)
	{
		cookie.secure = true;
	}

	return cookie;
};

// Populates a dialog from an element
WebDeveloper.Generated.populateDialogFromElement = function(cookieElement)
{
	var expires	= $(".web-developer-expires", cookieElement).text();

	$("#cookie-host").val($(".web-developer-host", cookieElement).text());
	$("#cookie-name").val($(".web-developer-name", cookieElement).text());
	$("#cookie-path").val($(".web-developer-path", cookieElement).text());
	$("#cookie-value").val($(".web-developer-value", cookieElement).text());

	// If this is a session cookie
	if(expires == WebDeveloper.Generated.storedLocale.atEndOfSession)
	{
		$("#cookie-expires").val("").prop("disabled", true);
		$("#cookie-session").prop("checked", true);
	}
	else
	{
		$("#cookie-expires").val(expires).prop("disabled", false);
		$("#cookie-session").prop("checked", false);
	}

	// If this is not a secure cookie
	if($(".web-developer-secure", cookieElement).text() == WebDeveloper.Generated.storedLocale.no)
	{
		$("#cookie-secure").prop("checked", false);
	}
	else
	{
		$("#cookie-secure").prop("checked", true);
	}
};

// Populates an element from a cookie
WebDeveloper.Generated.populateElementFromCookie = function(cookieElement, cookie)
{
	$(".web-developer-host", cookieElement).text(cookie.host);
	$(".web-developer-name", cookieElement).text(cookie.name);
	$(".web-developer-path", cookieElement).text(cookie.path);
	$(".web-developer-value", cookieElement).text(cookie.value);

	// If the cookie is secure
	if(cookie.secure)
	{
		$(".web-developer-secure", cookieElement).text(WebDeveloper.Generated.storedLocale.yes);
	}
	else
	{
		$(".web-developer-secure", cookieElement).text(WebDeveloper.Generated.storedLocale.no);
	}

	// If the cookie is a session cookie
	if(cookie.session)
	{
		$(".web-developer-expires", cookieElement).text(WebDeveloper.Generated.storedLocale.atEndOfSession);
	}
	else
	{
		$(".web-developer-expires", cookieElement).text(cookie.expires);
	}

	return cookie;
};

// Resets the edit cookie dialog
WebDeveloper.Generated.resetEditDialog = function(editDialog)
{
	$(".error", editDialog).removeClass("error");
	$(".help-inline", editDialog).html("");
};

// Shows the delete cookie dialog
WebDeveloper.Generated.showDeleteDialog = function()
{
	var cookieElement = $(this).parent();
	var cookieName		= $(".web-developer-name", cookieElement).text();
	var deleteDialog	= $("#delete-dialog");

	WebDeveloper.Generated.cookie = cookieElement;

	$(".alert").remove();

	$("h3", deleteDialog).html(WebDeveloper.Generated.storedLocale.deleteCookie);
	$("p", deleteDialog).html(WebDeveloper.Generated.storedLocale.deleteCookieConfirmation.replace("%S", "<strong>" + cookieName + "</strong>"));

	deleteDialog.modal("show");
};

// Shows the edit cookie dialog
WebDeveloper.Generated.showEditDialog = function(cookieElement)
{
	var editDialog = $("#edit-dialog");

	WebDeveloper.Generated.cookie = cookieElement;

	$(".alert").remove();

	$("h3", editDialog).html(WebDeveloper.Generated.storedLocale.editCookie);
	WebDeveloper.Generated.populateDialogFromElement(cookieElement);
	WebDeveloper.Generated.resetEditDialog(editDialog);

	editDialog.modal("show");
};

// Returns true if the edit cookie dialog is valid
WebDeveloper.Generated.validateEditDialog = function()
{
	var expires = $("#cookie-expires");
	var host		= $("#cookie-host");
	var name		= $("#cookie-name");
	var path		= $("#cookie-path");
	var valid		= true;

	WebDeveloper.Generated.resetEditDialog($("#edit-dialog"));

	// If the cookie name is not set
	if(!name.val())
	{
		name.next().html(WebDeveloper.Generated.storedLocale.nameCannotBeEmpty);
		name.closest(".control-group").addClass("error");

		valid = false;
	}

	// If the cookie host is not set
	if(!host.val())
	{
		host.next().html(WebDeveloper.Generated.storedLocale.hostCannotBeEmpty);
		host.closest(".control-group").addClass("error");

		valid = false;
	}

	// If the cookie path is not set
	if(!path.val())
	{
		path.next().html(WebDeveloper.Generated.storedLocale.pathCannotBeEmpty);
		path.closest(".control-group").addClass("error");

		valid = false;
	}

	// If the cookie is not a session cookie
	if(!$("#cookie-session").prop("checked"))
	{
		var expiresValue = WebDeveloper.Common.trim(expires.val());

		// If the cookie expires is not set
		if(!expiresValue)
		{
			expires.next().html(WebDeveloper.Generated.storedLocale.expiresCannotBeEmpty);
			expires.closest(".control-group").addClass("error");

			valid = false;
		}
		else if(new Date(expiresValue) == "Invalid Date")
		{
			expires.next().html(WebDeveloper.Generated.storedLocale.expiresNotValid);
			expires.closest(".control-group").addClass("error");

			valid = false;
		}
	}

	return valid;
};
var WebDeveloper = WebDeveloper || {};

WebDeveloper.Cookies = WebDeveloper.Cookies || {};

// Adds a cookie
WebDeveloper.Cookies.addCookie = function(cookie)
{
	var path						 = WebDeveloper.Common.trim(cookie.path);
	var cookieString		 = WebDeveloper.Common.trim(cookie.name) + "=" + WebDeveloper.Common.trim(cookie.value) + ";path=" + path + ";";
	var cookiePreference = WebDeveloper.Preferences.getIntegerPreference("network.cookie.cookieBehavior");
	var host						 = WebDeveloper.Common.trim(cookie.host);
	var scheme					 = "http://";
	var uri							 = null;

	// If the host is a domain
	if(host.charAt(0) == ".")
	{
		cookieString += "domain=" + host + ";";
		host					= host.substring(1);
	}

	// If this is not a session cookie
	if(!cookie.session)
	{
		cookieString += "expires=" + new Date(WebDeveloper.Common.trim(cookie.expires)).toUTCString() + ";";
	}

	// If the cookie is secure
	if(cookie.secure)
	{
		cookieString += "secure;";
		scheme				= "https://";
	}

	uri = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(scheme + host + path, null, null);

	// If the cookie preference is not allowing all cookies
	if(cookiePreference !== 0)
	{
		WebDeveloper.Preferences.setIntegerPreference("network.cookie.cookieBehavior", 0);
	}

	Components.classes["@mozilla.org/cookieService;1"].getService().QueryInterface(Components.interfaces.nsICookieService).setCookieString(uri, null, cookieString.substring(0, cookieString.length - 1), null);

	// If the cookie preference was not allowing all cookies
	if(cookiePreference !== 0)
	{
		WebDeveloper.Preferences.setIntegerPreference("network.cookie.cookieBehavior", cookiePreference);
	}
};

// Returns true if you can edit a local cookie
WebDeveloper.Cookies.canEditLocalCookie = function()
{
	return true;
};

// Deletes a cookie
WebDeveloper.Cookies.deleteCookie = function(cookie)
{
	Components.classes["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager2).remove(cookie.host, cookie.name, cookie.path, false);
};

// Returns all cookies
WebDeveloper.Cookies.getAllCookies = function()
{
	var allCookies				= [];
	var cookie						= null;
	var cookieEnumeration = Components.classes["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager2).enumerator;
	var cookieObject			= null;

	// Loop through the cookies
	while(cookieEnumeration.hasMoreElements())
	{
		cookie			 = {};
		cookieObject = cookieEnumeration.getNext().QueryInterface(Components.interfaces.nsICookie2);

		cookie.expires	= cookieObject.expires;
		cookie.host			= cookieObject.host;
		cookie.httpOnly = cookieObject.isHttpOnly;
		cookie.name			= cookieObject.name;
		cookie.path			= cookieObject.path;
		cookie.secure		= cookieObject.isSecure;
		cookie.session	= cookieObject.isSession;
		cookie.value		= cookieObject.value;

		allCookies.push(cookie);
	}

	return allCookies;
};
